from datetime import datetime, timedelta
from itertools import pairwise

import pytest

from conftest import create_daily_schedule

pytestmark = pytest.mark.django_db


def url(include_virtual=False, start=None, end=None):
    if include_virtual and start and end:
        return f"/api/v1/program/playout/?include_virtual=true&start={start}&end={end}"
    elif start and end:
        return f"/api/v1/program/playout/?start={start}&end={end}"
    elif include_virtual:
        return "/api/v1/program/playout/?include_virtual=true"
    else:
        return "/api/v1/program/playout/"


def assert_entry(entry, show) -> None:
    """asserts the playout entry corresponds to the given show."""

    assert entry["episode"]
    assert entry["schedule"]
    assert entry["show"]
    assert entry["timeslot"]
    assert entry["timeslotId"]

    assert entry["showId"] == entry["show"]["id"] == show.id
    assert entry["show"]["name"] == show.name


def assert_virtual_entry(entry, fallback_show) -> None:
    """asserts the playout entry is virtual and corresponds to given fallback show."""

    assert entry["show"]
    assert not entry["episode"]
    assert not entry["schedule"]
    assert not entry["timeslot"]
    assert not entry["timeslotId"]

    assert entry["showId"] == entry["show"]["id"] == fallback_show.id
    assert entry["show"]["name"] == fallback_show.name


def test_playout(admin_api_client, api_client, daily_rrule, show):
    create_daily_schedule(admin_api_client, daily_rrule, show)

    response = api_client.get(url())

    assert response.status_code == 200
    assert len(response.json()) == 1

    assert_entry(response.json()[0], show)


def test_playout_one_week(admin_api_client, api_client, daily_rrule, show):
    create_daily_schedule(admin_api_client, daily_rrule, show)

    now = datetime.now()
    in_one_week = now + timedelta(days=7)

    response = api_client.get(url(start=now.isoformat(), end=in_one_week.isoformat()))

    assert response.status_code == 200
    assert len(response.json()) == 6 or 7  # I’m not sure why, but this changes around midnight.

    for entry in response.json():
        assert_entry(entry, show)


def test_playout_include_virtual(
    admin_api_client,
    api_client,
    daily_rrule,
    show,
    fallback_show,
    radio_settings,
):
    create_daily_schedule(admin_api_client, daily_rrule, show)

    response = api_client.get(url(include_virtual=True))

    assert response.status_code == 200
    assert len(response.json()) == 3

    virtual_entry1, entry, virtual_entry2 = response.json()

    assert_virtual_entry(virtual_entry1, fallback_show)
    assert_entry(entry, show)
    assert_virtual_entry(virtual_entry2, fallback_show)

    assert virtual_entry1["end"] == entry["start"]
    assert entry["end"] == virtual_entry2["start"]


def test_playout_one_week_include_virtual(
    admin_api_client,
    api_client,
    daily_rrule,
    show,
    fallback_show,
    radio_settings,
):
    create_daily_schedule(admin_api_client, daily_rrule, show)

    now = datetime.now()
    in_one_week = now + timedelta(days=7)

    response = api_client.get(
        url(include_virtual=True, start=now.isoformat(), end=in_one_week.isoformat())
    )

    assert response.status_code == 200
    assert len(response.json()) == 13 or 15  # I’m not sure why, but this changes around midnight.

    entries = response.json()

    for virtual_entry in entries[0::2]:
        assert_virtual_entry(virtual_entry, fallback_show)

    for entry in entries[1::2]:
        assert_entry(entry, show)

    for entry1, entry2 in pairwise(entries):
        assert entry1["end"] == entry2["start"]
