from datetime import date, datetime, timedelta
from typing import Any

import pytest

from conftest import assert_data
from program.models import TimeSlot
from program.tests.factories import RRuleFactory, ScheduleFactory

pytestmark = pytest.mark.django_db


def url(schedule=None) -> str:
    if schedule:
        return f"/api/v1/schedules/{schedule.id}/"
    else:
        return "/api/v1/schedules/"


def schedule_data(rrule, show) -> dict[str, dict[str, str | Any]]:
    now = datetime.now()
    in_an_hour = now + timedelta(hours=1)
    in_a_year = now + timedelta(days=365)

    return {
        "schedule": {
            "end_time": in_an_hour.strftime("%H:%M:%S"),
            "first_date": now.strftime("%Y-%m-%d"),
            "last_date": in_a_year.strftime("%Y-%m-%d"),
            "rrule_id": rrule.id,
            "show_id": show.id,
            "start_time": now.strftime("%H:%M:%S"),
        },
    }


def test_create_once_schedule(admin_api_client, once_rrule, show):
    data = schedule_data(once_rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 1

    assert_data(response, data)


def test_create_daily_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=3)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 365 or 366

    assert_data(response, data)


def test_create_business_days_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=2, by_weekdays="0,1,2,3,4")
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 261 or 262

    assert_data(response, data)


def test_create_weekends_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=2, by_weekdays="5,6")
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 104 or 105

    assert_data(response, data)


def test_create_weekly_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=2)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 52 or 53

    assert_data(response, data)


def test_create_2weekly_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=2, interval=2)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 26 or 27

    assert_data(response, data)


def test_create_4weekly_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=2, interval=4)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 13 or 14

    assert_data(response, data)


def test_create_monthly_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=1)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 12 or 13

    assert_data(response, data)


def test_create_2monthly_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=1, interval=2)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 6 or 7

    assert_data(response, data)


def test_create_3monthly_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=1, interval=3)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 4 or 5

    assert_data(response, data)


def test_create_4monthly_schedule(admin_api_client, show):
    rrule = RRuleFactory(freq=1, interval=4)
    data = schedule_data(rrule, show)

    response = admin_api_client.post(url(), data=data, format="json")

    assert response.status_code == 201
    assert TimeSlot.objects.all().count() == 3 or 4

    assert_data(response, data)


def test_create_schedule_forbidden_for_common_user(common_api_client1, once_rrule, show):
    data = schedule_data(once_rrule, show)

    response = common_api_client1.post(url(), data=data, format="json")

    assert response.status_code == 403


def test_delete_schedule(admin_api_client, once_schedule):
    response = admin_api_client.delete(url(schedule=once_schedule))

    assert response.status_code == 204


def test_delete_schedule_forbidden_for_common_user(common_api_client1, once_schedule):
    response = common_api_client1.delete(url(schedule=once_schedule))

    assert response.status_code == 403


def test_list_schedules(api_client, once_rrule, show):
    SCHEDULES = 3
    ScheduleFactory.create_batch(size=SCHEDULES, show=show, rrule=once_rrule)

    response = api_client.get(url())

    assert response.status_code == 200
    assert len(response.data) == 3


def test_retrieve_schedule(api_client, once_schedule):
    response = api_client.get(url(schedule=once_schedule))

    assert response.status_code == 200


def test_update_schedule(admin_api_client, once_schedule):
    update = {
        "schedule": {
            "first_date": once_schedule.first_date,
            "start_time": once_schedule.start_time,
            "end_time": once_schedule.end_time,
            "rrule_id": once_schedule.rrule.id,
        },
    }

    response = admin_api_client.put(url(schedule=once_schedule), data=update, format="json")

    assert response.status_code == 200

    assert_data(response, update)


def test_patch_set_default_playlist_id(admin_api_client, once_schedule):
    update = {"default_playlist_id": 42}

    response = admin_api_client.patch(url(schedule=once_schedule), data=update)

    assert response.status_code == 200
    assert response.data["default_playlist_id"] == update["default_playlist_id"]


def test_patch_clear_default_playlist_id(admin_api_client, once_schedule):
    update = {"default_playlist_id": None}

    response = admin_api_client.patch(url(schedule=once_schedule), data=update, format="json")

    assert response.status_code == 200
    assert response.data["default_playlist_id"] == update["default_playlist_id"]


def test_patch_set_is_repetition_true(admin_api_client, once_schedule):
    update = {"is_repetition": "true"}

    response = admin_api_client.patch(url(schedule=once_schedule), data=update)

    assert response.status_code == 200
    assert response.data["is_repetition"] is True


def test_patch_set_is_repetition_false(admin_api_client, once_schedule):
    update = {"is_repetition": "false"}

    response = admin_api_client.patch(url(schedule=once_schedule), data=update)

    assert response.status_code == 200
    assert response.data["is_repetition"] is False


def test_patch_last_date_delete_timeslots(admin_api_client, show_once_timeslot):
    update = {"last_date": (date.today() - timedelta(days=1)).isoformat()}

    response = admin_api_client.patch(url(schedule=show_once_timeslot.schedule), data=update)

    assert response.status_code == 200
    assert response.data["last_date"] == update["last_date"]
    assert show_once_timeslot.schedule.timeslots.all().count() == 0


def test_patch_last_date(admin_api_client, show_once_timeslot):
    update = {"last_date": (date.today() + timedelta(days=1)).isoformat()}

    response = admin_api_client.patch(url(schedule=show_once_timeslot.schedule), data=update)

    assert response.status_code == 200
    assert response.data["last_date"] == update["last_date"]
    assert show_once_timeslot.schedule.timeslots.all().count() == 1


def test_patch_last_date_bad_request(admin_api_client, once_schedule):
    once_schedule.last_date = date.today() + timedelta(days=1)
    once_schedule.save()

    update = {"last_date": (date.today() + timedelta(days=2)).isoformat()}

    response = admin_api_client.patch(url(schedule=once_schedule), data=update)

    assert response.status_code == 400
